// SPDX-License-Identifier: Zlib
/* ------------------------------------------------------------------------- */
/*!
 *  \file       mgl_audio_stream_queue.h
 *  \brief      MGL オーディオストリームキュー
 *  \date       Since: January 29, 2021. 17:20:41 JST.
 *  \author     Acerola
 */
/* ------------------------------------------------------------------------- */

#ifndef INCGUARD_MGL_AUDIO_STREAM_QUEUE_H_1611908441
#define INCGUARD_MGL_AUDIO_STREAM_QUEUE_H_1611908441

#include <mgl/stl/mgl_stl_memory.h>
#include <mutex>
#include <condition_variable>

namespace MGL::Audio
{
//! オーディオストリームキュークラス
class StreamQueue
{
public:
    //! キューバッファ情報
    struct QueueBuffer
    {
        STL::unique_ptr<float[]> data;  //!< バッファアドレス
        uint32_t channelCount;          //!< 1フレームのチャンネル数（1: モノラル，2: ステレオ）
        size_t dataSize;                //!< データのサイズ
        QueueBuffer *next;              //!< 次のバッファアドレス
        bool isActive;                  //!< 使用中フラグ

        //! 初期化用コンストラクタ
        QueueBuffer() noexcept
            : data(nullptr)
            , channelCount(0)
            , dataSize(0)
            , next(nullptr)
            , isActive(false)
        {}
    };

    StreamQueue(size_t bufferSize, uint32_t bufferCount) noexcept;

    void Reset() noexcept;
    
    QueueBuffer *GetFreeBuffer() noexcept;
    bool AddBuffer(QueueBuffer *queueBuffer) noexcept;
    bool GetSample(float &frameDataL, float &frameDataR) noexcept;
    
    void Wait() noexcept;
    
    /* ------------------------------------------------------------------------- */
    /*!
     *  \brief      バッファサイズを取得
     *  \return     バッファサイズ
     */
    /* ------------------------------------------------------------------------- */
    constexpr size_t GetBufferSize() const noexcept
    {
        return _bufferSize;
    }
    
    
private:
    STL::unique_ptr<QueueBuffer[]> _bufferArray;
    QueueBuffer *_top;
    QueueBuffer *_tail;

    size_t _bufferSize;
    uint32_t _bufferCount;
    uint32_t _activeBufferCount;
    uint32_t _currentFrameOffset;

    std::mutex _mutex;
    std::condition_variable _condition;
};
}   // namespace MGL::Audio

#endif	// INCGUARD_MGL_AUDIO_STREAM_QUEUE_H_1611908441

// vim: et ts=4 sw=4 sts=4
