// SPDX-License-Identifier: Zlib
/* ------------------------------------------------------------------------- */
/*!
 *  \file       mgl_file_delegate_cocoa.h
 *  \brief      MGL iOS/tvOS用ファイルデリゲート
 *  \date       Since: February 27, 2021. 19:02:24 JST.
 *  \author     Acerola
 *  \note
 *      iOS/tvOSは基本的にPOSIX準拠のAPIが利用できるが，一部の機能は利用できないため，
 *      それらを置き換えるためにPOSIX用デリゲートから派生させている．
 */
/* ------------------------------------------------------------------------- */

#ifndef INCGUARD_MGL_FILE_DELEGATE_COCOA_H_1614420144
#define INCGUARD_MGL_FILE_DELEGATE_COCOA_H_1614420144

#include <mgl/mgl_environment.h>
#if defined(MGL_FILE_DELEGATE_ENABLE_POSIX) && defined(MGL_FILE_DELEGATE_ENABLE_COCOA)

#include <mgl/file/delegate/mgl_file_delegate_posix.h>

namespace MGL::File
{
class CocoaDelegate : public POSIXDelegate
{
public:
    //! このデリゲートを表すデリゲートキー
    inline static constexpr DelegateKey kDelegateKey = MakeDelegateKey("MGL-Cocoa");

    virtual Result MakeDirectory(MountWork *mountWork, const char *path) noexcept override;
};
}   // namespace MGL::File

#endif  // defined(MGL_FILE_DELEGATE_ENABLE_POSIX) && defined(MGL_FILE_DELEGATE_ENABLE_COCOA)
#endif	// INCGUARD_MGL_FILE_DELEGATE_COCOA_H_1614420144

// vim: et ts=4 sw=4 sts=4
