// SPDX-License-Identifier: Zlib
/* ------------------------------------------------------------------------- */
/*!
 *  \file       mgl_file_throwing_utility.h
 *  \brief      MGL 例外を発生させるファイルユーティリティ
 *  \date       Since: February 5, 2021. 16:22:08 JST.
 *  \author     Acerola
 */
/* ------------------------------------------------------------------------- */

#ifndef INCGUARD_MGL_FILE_THROWING_UTILITY_H_1612509728
#define INCGUARD_MGL_FILE_THROWING_UTILITY_H_1612509728

#include <mgl/file/mgl_file_mounter.h>

namespace MGL::File
{
//! 例外を発生させるファイルユーティリティクラス
class ThrowingUtility
{
public:
    size_t GetSize(const char *path) const;
    void MakeDirectory(const char *path) const;
    void Move(const char *sourcePath, const char *destPath) const;
    void Remove(const char *path) const;
    void Copy(const char *sourcePath, const char *destPath) const;
    bool Exists(const char *path) const;
    bool IsSystemNativeFile(const char *path) const;
    void Mount(const char *mountName, const char *path, MountAccessType accessType, DelegateKey delegateKey = Mounter::kDefaultDelegateKey) const;
    void Unmount(const char *mountName) const;
    void Remount(const char *mountName, const char *path, MountAccessType accessType, DelegateKey delegateKey = Mounter::kDefaultDelegateKey) const;
    bool IsMounted(const char *mountName) const noexcept;
    STL::string GetSystemNativePath(const char *path) const;
    void RemoveDelegate(DelegateKey key);
    void SetDefaultDelegate(DelegateKey key) noexcept;

    /* ------------------------------------------------------------------------- */
    /*!
     *  \brief      デリゲートの追加
     *  \tparam     DelegateClass       追加するデリゲートクラス
     *  \param[in]  key                 デリゲートキー
     *  \param[in]  args                デリゲートクラスのコンストラクタに渡す引数
     *  \exception  MGL::File::Result   失敗時にエラーが設定されたリザルトを返す
     */
    /* ------------------------------------------------------------------------- */
    template <class DelegateClass, class... Args>
    constexpr void AddDelegate(DelegateKey key, Args... args) const
    {
        auto result = Mounter::GetInstance().AddDelegate(key, STL::make_shared<DelegateClass>(args...));
        if (result.HasError())
        {
            throw result;
        }
    }
};
}   // namespace MGL::File
#endif	// INCGUARD_MGL_FILE_THROWING_UTILITY_H_1612509728

// vim: et ts=4 sw=4 sts=4
