// SPDX-License-Identifier: Zlib
/* ------------------------------------------------------------------------- */
/*!
 *  \file       mgl_gamepad_defs.h
 *  \brief      ゲームパッド関連各種定義
 *  \date       Since: September 24, 2022. 16:46:52 JST.
 *  \author     Acerola
 */
/* ------------------------------------------------------------------------- */

#ifndef INCGUARD_MGL_GAMEPAD_DEFS_H_1664005612
#define INCGUARD_MGL_GAMEPAD_DEFS_H_1664005612

#include <mgl/common/mgl_bit.h>
#include <mgl/stl/mgl_stl_string.h>

#include <type_traits>
#include <string>

namespace MGL::Input
{
//! ゲームパッドのIDを表す型
enum class PadID : uint32_t
{
    Invalid,        //!< 無効値
    Any,            //!< 入力のあるいずれかのパッドを表す値
    UniqueIDStart,  //!< ユニークIDの開始値
    UniqueIDEnd = ~static_cast<std::underlying_type<PadID>::type>(0),    //!< ユニークIDの終了値
};

//! ゲームパッドのボタンのフラグ型
using PadButtonFlags = uint32_t;

//! ゲームパッドのボタンの定義
enum class PadButton : PadButtonFlags
{
    // 基本
    None        = 0,                            //!< 入力なし
    Up          = MakeBit<PadButtonFlags>(0),   //!< 上
    Down        = MakeBit<PadButtonFlags>(1),   //!< 下
    Left        = MakeBit<PadButtonFlags>(2),   //!< 左
    Right       = MakeBit<PadButtonFlags>(3),   //!< 右
    AnalogUp    = MakeBit<PadButtonFlags>(4),   //!< アナログ入力の上
    AnalogDown  = MakeBit<PadButtonFlags>(5),   //!< アナログ入力の下
    AnalogLeft  = MakeBit<PadButtonFlags>(6),   //!< アナログ入力の左
    AnalogRight = MakeBit<PadButtonFlags>(7),   //!< アナログ入力の右
    Button01    = MakeBit<PadButtonFlags>(8),   //!< ボタン1
    Button02    = MakeBit<PadButtonFlags>(9),   //!< ボタン2
    Button03    = MakeBit<PadButtonFlags>(10),  //!< ボタン3
    Button04    = MakeBit<PadButtonFlags>(11),  //!< ボタン4
    Button05    = MakeBit<PadButtonFlags>(12),  //!< ボタン5
    Button06    = MakeBit<PadButtonFlags>(13),  //!< ボタン6
    Button07    = MakeBit<PadButtonFlags>(14),  //!< ボタン7
    Button08    = MakeBit<PadButtonFlags>(15),  //!< ボタン8
    Button09    = MakeBit<PadButtonFlags>(16),  //!< ボタン9
    Button10    = MakeBit<PadButtonFlags>(17),  //!< ボタン10
    Button11    = MakeBit<PadButtonFlags>(18),  //!< ボタン11
    Button12    = MakeBit<PadButtonFlags>(19),  //!< ボタン12
    Button13    = MakeBit<PadButtonFlags>(20),  //!< ボタン13
    Button14    = MakeBit<PadButtonFlags>(21),  //!< ボタン14
    Button15    = MakeBit<PadButtonFlags>(22),  //!< ボタン15
    Button16    = MakeBit<PadButtonFlags>(23),  //!< ボタン16
    Button17    = MakeBit<PadButtonFlags>(24),  //!< ボタン17
    Button18    = MakeBit<PadButtonFlags>(25),  //!< ボタン18
    Button19    = MakeBit<PadButtonFlags>(26),  //!< ボタン19
    Button20    = MakeBit<PadButtonFlags>(27),  //!< ボタン20
    Button21    = MakeBit<PadButtonFlags>(28),  //!< ボタン21
    Button22    = MakeBit<PadButtonFlags>(29),  //!< ボタン22
    Cancel      = MakeBit<PadButtonFlags>(30),  //!< キャンセル
    Decide      = MakeBit<PadButtonFlags>(31),  //!< 決定

    // 共通の別名
    L1              = Button05,     //!< L1/LB
    R1              = Button06,     //!< R1/RB
    L2              = Button07,     //!< L2/LT
    R2              = Button08,     //!< R2/RT
    PrimaryMenu     = Button09,     //!< Pause/Start/Menu
    SecondaryMenu   = Button10,     //!< Select/Back/View
    Home            = Button11,     //!< ホームボタン
    L3              = Button12,     //!< L3/左サムスティックボタン
    R3              = Button13,     //!< R3/右サムスティックボタン
    Other1          = Button14,     //!< その他1
    Other2          = Button15,     //!< その他2
    Other3          = Button16,     //!< その他3
    Other4          = Button17,     //!< その他4
    Other5          = Button18,     //!< その他5
    Other6          = Button19,     //!< その他6
    Other7          = Button20,     //!< その他7
    Other8          = Button21,     //!< その他8
    Other9          = Button22,     //!< その他9
    
    // ダイヤモンド配置のボタン
    DiaDown     = Button01,         //!< ダイヤ配置の下ボタン
    DiaRight    = Button02,         //!< ダイヤ配置の右ボタン
    DiaLeft     = Button03,         //!< ダイヤ配置の左ボタン
    DiaUp       = Button04,         //!< ダイヤ配置の上ボタン

    // 任天堂系のボタン
    NintendoA   = DiaRight,         //!< 任天堂のAボタン
    NintendoB   = DiaDown,          //!< 任天堂のBボタン
    NintendoX   = DiaUp,            //!< 任天堂のXボタン
    NintendoY   = DiaLeft,          //!< 任天堂のYボタン

    // XInputのボタン
    XInputA         = DiaDown,          //!< XInputのAボタン
    XInputB         = DiaRight,         //!< XInputのBボタン
    XInputX         = DiaLeft,          //!< XInputのXボタン
    XInputY         = DiaUp,            //!< XInputのYボタン
    XInputMenu      = PrimaryMenu,      //!< XInputのMenu/Startボタン
    XInputView      = SecondaryMenu,    //!< XInputのView/Backボタン
    XInputShare     = Other1,           //!< XInputのShareボタン
    XInputPaddle1   = Other2,           //!< XInputの背面パドル1
    XInputPaddle2   = Other3,           //!< XInputの背面パドル2
    XInputPaddle3   = Other4,           //!< XInputの背面パドル3
    XInputPaddle4   = Other5,           //!< XInputの背面パドル4

    // DualShock/DualSenseのボタン
    DSCircle    = DiaRight,         //!< DualShock/DualSenseの◯ボタン
    DSCross     = DiaDown,          //!< DualShock/DualSenseの×ボタン
    DSTriangle  = DiaLeft,          //!< DualShock/DualSenseの△ボタン
    DSSquare    = DiaUp,            //!< DualShock/DualSenseの□ボタン
    DSOptions   = PrimaryMenu,      //!< DualShock/DualSenseのOptionsボタン
    DSShare     = SecondaryMenu,    //!< DualShock/DualSenseのShare/Createボタン
    DSTouchPad  = Other1,           //!< DualShock/DualSenseのタッチパッドボタン

    // MFi配置のボタン
    MFiA        = DiaDown,          //!< MFi配置のAボタン
    MFiB        = DiaRight,         //!< MFi配置のBボタン
    MFiX        = DiaLeft,          //!< MFi配置のXボタン
    MFiY        = DiaUp,            //!< MFi配置のYボタン
    
    // Apple Remote/Siri Remoteのボタン
    MFiTouchSurface =   Other1,     //!< タッチサーフェスのタッチ
};

//! ゲームパッドボタンのor演算の結果をフラグ型に変換するためのオペレータ（PadButton | PadButton）
constexpr PadButtonFlags operator|(PadButton lhs, PadButton rhs) noexcept
{
    return static_cast<PadButtonFlags>(lhs) | static_cast<PadButtonFlags>(rhs);
}

//! ゲームパッドボタンのor演算の結果をフラグ型に変換するためのオペレータ（PadButtonFlags | PadButton）
constexpr PadButtonFlags operator|(PadButtonFlags lhs, PadButton rhs) noexcept
{
    return lhs | static_cast<PadButtonFlags>(rhs);
}

//! ゲームパッドボタンのor演算の結果をフラグ型に変換するためのオペレータ （PadButtonFlags |= PadButton）
constexpr PadButtonFlags operator|=(PadButtonFlags &lhs, const PadButton &rhs) noexcept
{
    return lhs | static_cast<PadButtonFlags>(rhs);
}

//! ゲームパッドの全てのボタンを表す定数
constexpr PadButtonFlags kGamepadButtonAll = 0xFFFFFFFF;

//! ゲームパッドの入力フラグの最大数
constexpr size_t kGamepadFlagSize = 32;

//! ゲームパッドのボタンの最大数
constexpr size_t kGamepadButtonMax = 22;

//! ゲームパッドの種類
enum class PadType
{
    Disable,        //!< 無効/未接続
    MFiExtended,    //!< MFi拡張
    MFiMicro,       //!< AppleRemote/SiriRemote
    NintendoSwitch, //!< Nintend Switch
    DualShock4,     //!< DualShock4
    DualSense,      //!< DualSense
    XboxOne,        //!< Xbox One
    GenericHID,     //!< 汎用HID
    XInput,         //!< XInput
    DirectInput,    //!< DirectInput
    Other,          //!< その他
};

//! パッドのエントリータイプ
enum class PadEntry
{
    Player1 = 0,    //!< プレイヤー1
    Player2,        //!< プレイヤー2
    Player3,        //!< プレイヤー3
    Player4,        //!< プレイヤー4
    Player5,        //!< プレイヤー5
    Player6,        //!< プレイヤー6
    Player7,        //!< プレイヤー7
    Player8,        //!< プレイヤー8

    // 以下予約
    Reserve_Start,                          //!< 予約開始位置
    Reserve_MaxPlayerCount = Reserve_Start, //!< エントリー可能なプレイヤーの最大数
    Reserve_NoEntry,                        //!< エントリーなしの指定
    Reserve_Any,                            //!< 全ての指定
    Reserve_Auto,                           //!< 自動指定

    // 特殊なプレイヤー指定
    NoEntry = Reserve_NoEntry,  //!< エントリーなしを取得
    Any     = Reserve_Any,      //!< 全てを取得
    Auto    = Reserve_Auto,     //!< エントリー済みを優先して取得し，無ければNoEntryと同じ
};

//! ゲームパッドの最大エントリー数
constexpr size_t kGamepadEntryMax = static_cast<size_t>(PadEntry::Reserve_MaxPlayerCount);

//! パッドのプライオリティ
enum class PadPriority
{
    Low,    //!< 低
    High    //!< 高
};

//! ゲームパッドのデバイス情報
struct PadDeviceInfo
{
    STL::string vendorName;     //!< 製造元の名前
    STL::string productName;    //!< 製品名
    uint16_t vendorID;          //!< 製造元のID
    uint16_t productID;         //!< 製品ID

    //! コンストラクタ
    PadDeviceInfo() noexcept
        : vendorName()
        , productName()
        , vendorID()
        , productID()
    {
    }
};

}   // namespace MGL::Input

#endif	// INCGUARD_MGL_GAMEPAD_DEFS_H_1664005612

// vim: et ts=4 sw=4 sts=4
