// SPDX-License-Identifier: Zlib
/* ------------------------------------------------------------------------- */
/*!
 *  \file       mgl_leaderboard_defs.h
 *  \brief      MGL リーダーボード関連各種定義
 *  \date       Since: September 4, 2021. 1:32:07 JST.
 *  \author     Acerola
 */
/* ------------------------------------------------------------------------- */

#ifndef INCGUARD_MGL_LEADERBOARD_DEFS_H_1630686727
#define INCGUARD_MGL_LEADERBOARD_DEFS_H_1630686727

#include <cstdint>
#include <mgl/leaderboard/mgl_leaderboard_clipdata.h>
#include <mgl/stl/mgl_stl_containers.h>
#include <mgl/stl/mgl_stl_string.h>

namespace MGL::Leaderboard
{
//! リーダーボード用の識別子
using Identifier = uint32_t;

//! 無効な識別子を表す定数
constexpr Identifier kInvalidIdentifier = 0xFFFFFFFF;

//! オプションデータ配列の型
using OptionDataArray = STL::vector<uint32_t>;

//! ソートタイプ
enum class SortType
{
    Descending, //!< 降順（大→小）
    Ascending   //!< 昇順（小→大）
};

//! プレイヤーのスコープ
enum class PlayerScope
{
    Global,     //!< 全体
    Friend      //!< フレンドのみ
};

//! エントリデータ
struct EntryData
{
    uint64_t playerID;              //!< プレイヤーの識別子
    int32_t rank;                   //!< 順位
    STL::string playerName;         //!< プレイヤー名
    int32_t score;                  //!< スコア
    STL::vector<uint32_t> options;  //!< オプション情報
    bool hasClip;                   //!< クリップデータを持っているか
    uint64_t clipID;                //!< クリップデータの識別子

    //! コンストラクタ
    EntryData() noexcept
        : playerID(0)
        , rank(0)
        , playerName("")
        , score(0)
        , options()
        , hasClip(false)
        , clipID(0)
    {
    }
};

//! エントリ取得結果
struct FetchResults
{
    bool isSucceeded;                   //!< 成功フラグ
    STL::vector<EntryData> entryData;   //!< 取得したエントリデータ
    bool isFirst;                       //!< エントリデータが先頭のものかのフラグ
    bool isLast;                        //!< エントリデータが末尾のものかのフラグ

    //! コンストラクタ
    FetchResults() noexcept
        : isSucceeded(false)
        , entryData()
        , isFirst(false)
        , isLast(false)
    {
    }
};

//! エントリ送信結果
struct ReportResults
{
    bool isSucceeded;       //!< 送信成功フラグ

    //! コンストラクタ
    constexpr ReportResults() noexcept
        : isSucceeded(false)
    {
    }
};
}  // namespace MGL::Leaderboard

#endif	// INCGUARD_MGL_LEADERBOARD_DEFS_H_1630686727

// vim: et ts=4 sw=4 sts=4
