// SPDX-License-Identifier: Zlib
/* ------------------------------------------------------------------------- */
/*!
 *  \file       mgl_ascii_font.h
 *  \brief      MGL ASCIIフォント
 *  \date       Since: May 26, 2021. 18:36:09 JST.
 *  \author     Acerola
 */
/* ------------------------------------------------------------------------- */

#ifndef INCGUARD_MGL_ASCII_FONT_H_1622021769
#define INCGUARD_MGL_ASCII_FONT_H_1622021769

#include <mgl/render/font/mgl_font_resource.h>
#include <mgl/render/mgl_renderer_2d.h>

namespace MGL::Render
{
//! ASCIIフォントクラス
class AsciiFont : public FontResource
{
public:
    AsciiFont(TextureKey textureKey, uint32_t characterWidth, uint32_t characterHeight, const Vector2 &sourceOffset = MGL::Vector2()) noexcept;
    AsciiFont(
            const char *imagePath,
            uint32_t characterWidth,
            uint32_t characterHeight,
            bool isAsync = true,
            const Vector2 &sourceOffset = MGL::Vector2(),
            TextureLoaderKey loaderKey = TextureStorage::kDefaultLoaderKey
    ) noexcept;
    AsciiFont(
            const void *imageData,
            size_t dataSize,
            uint32_t characterWidth,
            uint32_t characterHeight,
            bool isAsync = true,
            const Vector2 &sourceOffset = MGL::Vector2(),
            TextureLoaderKey loaderKey = TextureStorage::kDefaultLoaderKey
    ) noexcept;

    virtual Vector2 Print(const Vector2 position, const FontOption &fontOption, const char *text) noexcept override;

    /* ------------------------------------------------------------------------- */
    /*!
     *  \brief      このフォントリソースが有効かどうかを取得
     *  \retval     true    有効
     *  \retval     false   無効
     */
    /* ------------------------------------------------------------------------- */
    virtual bool IsValid() const noexcept override
    {
        return _texture.IsValid();
    }
    
private:
    const char *PrintLine(Vector2 position, const char *text, Renderer2D &renderer, const DrawOption2D &drawOption, const FontOption &fontOption) noexcept;

    Texture _texture;
    Vector2 _characterSize;
    Vector2 _sourceOffset;
};
}   // namespace MGL::Render

#endif	// INCGUARD_MGL_ASCII_FONT_H_1622021769

// vim: et ts=4 sw=4 sts=4
