// SPDX-License-Identifier: Zlib
/* ------------------------------------------------------------------------- */
/*!
 *  \file       mgl_chrono_defs.h
 *  \brief      MGL 日付と時間の定義
 *  \date       Since: February 24, 2021. 4:19:19 JST.
 *  \author     Acerola
 */
/* ------------------------------------------------------------------------- */

#ifndef INCGUARD_MGL_CHRONO_DEFS_H_1614107959
#define INCGUARD_MGL_CHRONO_DEFS_H_1614107959

#include <mgl/mgl_environment.h>
#include <cstdint>
#include <ctime>

namespace MGL::System
{
//! POSIX時間を表す型
using POSIXTime = time_t;

//! 日付と時刻
struct DateTime
{
    uint16_t year;  //!< 年
    uint8_t month;  //!< 月
    uint8_t day;    //!< 日
    uint8_t hour;   //!< 時間
    uint8_t minute; //!< 分
    uint8_t second; //!< 秒

    //! コンストラクタ
    constexpr DateTime() noexcept
        : year(1970)
        , month(1)
        , day(1)
        , hour(0)
        , minute(0)
        , second(0)
    {}

    /* ------------------------------------------------------------------------- */
    /*!
     *  \brief      POSIX時間から初期化
     *  \param[in]  posixTime   POSIX時間
     */
    /* ------------------------------------------------------------------------- */
    DateTime(POSIXTime posixTime) noexcept
    {
        // TODO:
#if defined(MGL_TARGET_WIN32)
        struct tm temp;
        gmtime_s(&temp, &posixTime);
        struct tm *time = &temp;
#else
        struct tm *time = gmtime(&posixTime);
#endif

        year = static_cast<uint16_t>(time->tm_year + 1900);
        month = static_cast<uint8_t>(time->tm_mon + 1);
        day = static_cast<uint8_t>(time->tm_mday);
        hour = static_cast<uint8_t>(time->tm_hour);
        minute = static_cast<uint8_t>(time->tm_min);
        second = static_cast<uint8_t>(time->tm_sec);
    }
};

//! Tickタイムを表す型
using TickTime = uint64_t;

}   // namespace MGL::System

#endif	// INCGUARD_MGL_CHRONO_DEFS_H_1614107959

// vim: et ts=4 sw=4 sts=4
