// SPDX-License-Identifier: Zlib
/* ------------------------------------------------------------------------- */
/*!
 *  \file       mgl_system_chrono.h
 *  \brief      MGL 日付と時間
 *  \date       Since: February 24, 2021. 5:14:19 JST.
 *  \author     Acerola
 */
/* ------------------------------------------------------------------------- */

#ifndef INCGUARD_MGL_SYSTEM_CHRONO_H_1614111259
#define INCGUARD_MGL_SYSTEM_CHRONO_H_1614111259

#include <mgl/system/mgl_system_module_set.h>
#include <mgl/mgl_environment.h>

namespace MGL::System
{
//! 日付と時間のクラス
class Chrono
{
public:
    /* ------------------------------------------------------------------------- */
    /*!
     *  \brief      コンストラクタ
     */
    /* ------------------------------------------------------------------------- */
    Chrono() noexcept
        : _chrono(ModuleSet::GetInstance().GetChronoModule())
    {}

    /* ------------------------------------------------------------------------- */
    /*!
     *  \brief      現在の協定世界時間を取得
     *  \return     現在の協定世界時間
     */
    /* ------------------------------------------------------------------------- */
    MGL_MAYBE_CONSTEXPR POSIXTime GetUTCTime() const noexcept
    {
        return _chrono.GetUTCTime();
    }

    /* ------------------------------------------------------------------------- */
    /*!
     *  \brief      現在のローカル時間を取得
     *  \return     現在のローカル時間
     */
    /* ------------------------------------------------------------------------- */
    MGL_MAYBE_CONSTEXPR POSIXTime GetLocalTime() const noexcept
    {
        return _chrono.GetLocalTime();
    }
    
    /* ------------------------------------------------------------------------- */
    /*!
     *  \brief      Tickタイマーに対応しているかを取得
     *  \retval     true    対応している
     *  \retval     false   対応していない
     */
    /* ------------------------------------------------------------------------- */
    MGL_MAYBE_CONSTEXPR bool IsAvailableTickTimer() const noexcept
    {
        return _chrono.IsAvailableTickTimer();
    }

    /* ------------------------------------------------------------------------- */
    /*!
     *  \brief      現在のTickTimeを取得
     *  \return     現在のTickTime
     */
    /* ------------------------------------------------------------------------- */
    MGL_MAYBE_CONSTEXPR TickTime GetTickTime() const noexcept
    {
        return _chrono.GetTickTime();
    }

    /* ------------------------------------------------------------------------- */
    /*!
     *  \brief      TickTimeをミリ秒に変換
     *  \return     変換後の時間
     */
    /* ------------------------------------------------------------------------- */
    MGL_MAYBE_CONSTEXPR float ConvertTickTimeToMillisecond(TickTime tickTime) const noexcept
    {
        return _chrono.ConvertTickTimeToNanosecond(tickTime) / 1000.0f / 1000.0f;
    }

    /* ------------------------------------------------------------------------- */
    /*!
     *  \brief      TickTimeをマイクロ秒に変換
     *  \return     変換後の時間
     */
    /* ------------------------------------------------------------------------- */
    MGL_MAYBE_CONSTEXPR float ConvertTickTimeToMicrosecond(TickTime tickTime) const noexcept
    {
        return _chrono.ConvertTickTimeToNanosecond(tickTime) / 1000.0f;
    }

    /* ------------------------------------------------------------------------- */
    /*!
     *  \brief      TickTimeをナノ秒に変換
     *  \return     変換後の時間
     */
    /* ------------------------------------------------------------------------- */
    MGL_MAYBE_CONSTEXPR float ConvertTickTimeToNanosecond(TickTime tickTime) const noexcept
    {
        return _chrono.ConvertTickTimeToNanosecond(tickTime);
    }

private:
    ChronoModuleInterface &_chrono;
};
}   // namespace MGL::System

#endif	// INCGUARD_MGL_SYSTEM_CHRONO_H_1614111259

// vim: et ts=4 sw=4 sts=4
