// SPDX-License-Identifier: Zlib
/* ------------------------------------------------------------------------- */
/*!
 *  \file       mgl_window_module_win32.h
 *  \brief      MGL Win32用ウィンドウモジュール
 *  \date       Since: March 29, 2021. 16:56:37 JST.
 *  \author     Acerola
 */
/* ------------------------------------------------------------------------- */

#ifndef INCGUARD_MGL_WINDOW_MODULE_WIN32_H_1617004597
#define INCGUARD_MGL_WINDOW_MODULE_WIN32_H_1617004597

#include <mgl/mgl_environment.h>
#if defined(MGL_TARGET_WIN32)

#include <mgl/system/window/mgl_window_module_interface.h>

namespace MGL::System
{
//! Win32用ウィンドウモジュール
class Win32WindowModule : public WindowModuleInterface
{
public:
    Win32WindowModule() noexcept;

    /* ------------------------------------------------------------------------- */
    /*!
     *  \brief      ウィンドウ機能が有効かを取得
     *  \retval     true    有効
     *  \retval     false   無効
     */
    /* ------------------------------------------------------------------------- */
    virtual bool IsAvailable() const noexcept
    {
        return true;
    }

    virtual void Resize(uint32_t width, uint32_t height) noexcept override;

    virtual void SetFullscreen(bool isEnabled) noexcept override;
    virtual void ToggleFullscreen() noexcept override;
    virtual bool IsFullscreen() const noexcept override;
    
    virtual void Focus() noexcept override;
    virtual bool IsFocused() const noexcept override;

    virtual void Alert(const char *message) const noexcept override;

    virtual void SetTitle(const char *title) noexcept override;

    virtual int32_t GetIdentify() const noexcept override;

    virtual ScreenSize GetScreenSize() const noexcept override;
};
}   // namespace MGL::System

#endif  // MGL_TARGET_WIN32
#endif	// INCGUARD_MGL_WINDOW_MODULE_WIN32_H_1617004597

// vim: et ts=4 sw=4 sts=4
