// SPDX-License-Identifier: Zlib
/* ------------------------------------------------------------------------- */
/*!
 *  \file       mgl_task_default.h
 *  \brief      MGL デフォルトタスク
 *  \date       Since: June 3, 2021. 17:03:51 JST.
 *  \author     Acerola
 */
/* ------------------------------------------------------------------------- */

#ifndef INCGUARD_MGL_TASK_DEFAULT_H_1622707431
#define INCGUARD_MGL_TASK_DEFAULT_H_1622707431

namespace MGL::Task
{
//! デフォルトの実行ステージ
enum
{
    kExecuteStageUpdate,    //!< 更新処理
    kExecuteStageRender,    //!< 描画処理
};

//! デフォルトの実行ステージ設定に対応したタスク
class DefaultTask : public Node
{
public:
    /* ------------------------------------------------------------------------- */
    /*!
     *  \brief      コンストラクタ
     *  \param[in]  identifier    タスクのID
     */
    /* ------------------------------------------------------------------------- */
    constexpr DefaultTask(Identifier identifier) noexcept
        : MGL::Task::Node(identifier)
    {}

    virtual ~DefaultTask() noexcept = default;

    /* ------------------------------------------------------------------------- */
    /*!
     *  \brief      実行時の処理
     *  \param[in]  stage   実行ステージ
     */
    /* ------------------------------------------------------------------------- */
    virtual void OnExecute(ExecuteStage stage) noexcept override
    {
        // UpdateステージではOnUpdate()を呼び出す
        if (stage == kExecuteStageUpdate)
        {
            OnUpdate();
        }
        // RenderステージではOnRender()を呼び出す
        else if (stage == kExecuteStageRender)
        {
            OnRender();
        }
    }

protected:
    /* ------------------------------------------------------------------------- */
    /*!
     *  \brief      更新処理
     */
    /* ------------------------------------------------------------------------- */
    virtual void OnUpdate() noexcept {}

    /* ------------------------------------------------------------------------- */
    /*!
     *  \brief      描画処理
     */
    /* ------------------------------------------------------------------------- */
    virtual void OnRender() noexcept {}
};

}   // namespace MGL::Task
#endif	// INCGUARD_MGL_TASK_DEFAULT_H_1622707431

// vim: et ts=4 sw=4 sts=4
