// SPDX-License-Identifier: Zlib
/* ------------------------------------------------------------------------- */
/*!
 *  \file       mgl_task_weak_node.h
 *  \brief      MGL 弱参照タスクノード
 *  \date       Since: June 3, 2021. 10:48:12 JST.
 *  \author     Acerola
 */
/* ------------------------------------------------------------------------- */

#ifndef INCGUARD_MGL_TASK_WEAK_NODE_H_1622684892
#define INCGUARD_MGL_TASK_WEAK_NODE_H_1622684892

#include <mgl/task/mgl_task_node.h>

namespace MGL::Task
{
//! 弱参照タスクノード
class WeakNode
{
public:
    /* ------------------------------------------------------------------------- */
    /*!
     *  \brief      コンストラクタ
     */
    /* ------------------------------------------------------------------------- */
    constexpr WeakNode() noexcept
        : _listIndex(0)
        , _identifier(0)
        , _uniqueId(UniqueIdentifier::Invalid)
    {}

    WeakNode(size_t listIndex) noexcept;
    WeakNode(const Node *node) noexcept;

    bool Set(size_t listIndex) noexcept;
    bool Set(const Node *node) noexcept;
    Node *Get() noexcept;

private:
    size_t _listIndex;
    Identifier _identifier;
    UniqueIdentifier _uniqueId;
};

/* ------------------------------------------------------------------------- */
/*!
 *  \brief      タスクノード間のキャスト用テンプレート
 *  \tparam     T           キャスト先のクラス
 *  \tparam     identifier  Tと関連付けられたタスクノードID
 *  \return     参照先のIDと引数のIDが一致していればキャストしたタクスノードを返し，そうで無ければnullptrを返す
 *  \note
 *      キャスト先のクラスに静的なメンバ変数kTaskIdentifyが定義されている場合，
 *      テンプレート引数のidentifyを省略することでそちらを参照する．
 */
/* ------------------------------------------------------------------------- */
template <class T, Identifier identifier = T::kTaskIdentifier>
T *NodeCast(WeakNode &weakNode) noexcept
{
    if (auto *node = weakNode.Get(); node != nullptr)
    {
        if (node->GetIdentifier() == identifier)
        {
            return reinterpret_cast<T *>(node);
        }
    }

    return nullptr;
}

};  // namespace MGL::Task

#endif	// INCGUARD_MGL_TASK_WEAK_NODE_H_1622684892

// vim: et ts=4 sw=4 sts=4
