// SPDX-License-Identifier: Zlib
/* ------------------------------------------------------------------------- */
/*!
 *  \file       mgl_text_index_converter.h
 *  \brief      MGL テキストのインデックス化コンバータ
 *  \date       Since: June 27, 2021. 5:31:10 JST.
 *  \author     Acerola
 */
/* ------------------------------------------------------------------------- */

#ifndef INCGUARD_MGL_TEXT_INDEX_CONVERTER_H_1624739470
#define INCGUARD_MGL_TEXT_INDEX_CONVERTER_H_1624739470

#include <mgl/text/mgl_text_defs.h>
#include <mgl/stl/mgl_stl_memory.h>

namespace MGL::Text
{
//! インデックスコンバータクラス
class IndexConverter
{
public:
    virtual ~IndexConverter() noexcept = default;

    /* ------------------------------------------------------------------------- */
    /*!
     *  \brief      文字をインデックス文字に変換
     *  \param[in]  character   変換元の文字（UTF-32）
     *  \return     インデックス文字
     */
    /* ------------------------------------------------------------------------- */
    virtual IndexedCharacter ToIndexedCharacter(char32_t character) const noexcept = 0;
    
    /* ------------------------------------------------------------------------- */
    /*!
      *  \brief      インデックス文字を通常の文字に変換
      *  \param[in]  indexedCharacter    インデックス文字
      *  \return     インデックスに対応したUTF-32エンコードの文字
      */
     /* ------------------------------------------------------------------------- */
    virtual char32_t ToCharacter(IndexedCharacter indexedCharacter) const noexcept = 0;

    STL::unique_ptr<Text::IndexedCharacter[]> ToIndexedString(const char *text) const noexcept;
};
}   // namespace MGL::Text

#endif	// INCGUARD_MGL_TEXT_INDEX_CONVERTER_H_1624739470

// vim: et ts=4 sw=4 sts=4
