// SPDX-License-Identifier: Zlib
/* ------------------------------------------------------------------------- */
/*!
 *  \file       mgl_locale_module_apple.mm
 *  \brief      MGL Apple（macOS/iOS/tvOS）ロケール情報モジュール
 *  \date       Since: February 23, 2021. 11:28:42 JST.
 *  \author     Acerola
 */
/* ------------------------------------------------------------------------- */

#include <mgl/system/locale/mgl_locale_module_apple.h>
#if defined(MGL_TARGET_MACOS) || defined(MGL_TARGET_IOS) || defined(MGL_TARGET_TVOS)

#include <Foundation/Foundation.h>

namespace MGL::System
{

namespace
{
// 言語テーブル
struct LanguageTable
{
    Language language;      // 言語
    const char *prefix;     // 判定用プレフィックス
};
constexpr LanguageTable kLanguageTable[] =
{
    {Language::English,     "en-"},     // 英語
    {Language::Japanese,    "ja-"},     // 日本語
};
}


/* ------------------------------------------------------------------------- */
/*!
 *  \brief      コンストラクタ
 */
/* ------------------------------------------------------------------------- */
AppleLocaleModule::AppleLocaleModule() noexcept
    : _currentInfo()
    , _defaultInfo()
{
}


/* ------------------------------------------------------------------------- */
/*!
 *  \brief      初期化
 */
/* ------------------------------------------------------------------------- */
void AppleLocaleModule::Initialize() noexcept
{
    // 言語の取得
    for (const NSString *language in NSLocale.preferredLanguages)
    {
        for (const auto tableElem : kLanguageTable)
        {
            if ([language hasPrefix:@(tableElem.prefix)])
            {
                _defaultInfo.language = tableElem.language;
                break;
            }
        }
        
        if (_defaultInfo.language != Language::Unknown)
        {
            break;
        }
    }
    
    // 数値の区切り文字を取得
    auto *locale = NSLocale.autoupdatingCurrentLocale;
    _defaultInfo.decimalSeparator = locale.decimalSeparator.UTF8String;
    _defaultInfo.groupingSeparator = locale.groupingSeparator.UTF8String;
    
    // 時差を取得
    _defaultInfo.timeDifferent = static_cast<int32_t>(NSTimeZone.localTimeZone.secondsFromGMT);
    
    _currentInfo = _defaultInfo;
}
}   // namespace MGL::System

#endif  // defined(MGL_TARGET_MACOS) || defined(MGL_TARGET_IOS) || defined(MGL_TARGET_TVOS)

// vim: et ts=4 sw=4 sts=4
