// SPDX-License-Identifier: Zlib
/* ------------------------------------------------------------------------- */
/*!
 *  \file       mgl_locale_module_win32.cc
 *  \brief      MGL Win32用ロケール情報モジュール
 *  \date       Since: March 29, 2021. 17:44:46 JST.
 *  \author     Acerola
 */
/* ------------------------------------------------------------------------- */

#include <mgl/system/locale/mgl_locale_module_win32.h>
#if defined(MGL_TARGET_WIN32)

#include <Windows.h>

namespace MGL::System
{
namespace
{
// 言語テーブル
struct LanguageTable
{
    Language language;      // 言語
    LANGID primaryLangId;   // プライマリの言語ID
    LANGID subLangId;       // サブ言語ID．SUBLANG_NEUTRALを指定した場合はチェックしない
};
constexpr LanguageTable kLanguageTable[] =
{
    {Language::English,   LANG_ENGLISH,     SUBLANG_NEUTRAL },     // 英語
    {Language::Japanese,  LANG_JAPANESE,    SUBLANG_NEUTRAL },     // 日本語
};
}
/* ------------------------------------------------------------------------- */
/*!
 *  \brief      コンストラクタ
 */
/* ------------------------------------------------------------------------- */
Win32LocaleModule::Win32LocaleModule() noexcept
    : _currentInfo()
    , _defaultInfo()
{
}


/* ------------------------------------------------------------------------- */
/*!
 *  \brief      初期化
 */
/* ------------------------------------------------------------------------- */
void Win32LocaleModule::Initialize() noexcept
{
    // 言語を取得
    auto langId = GetUserDefaultLangID();
    for (const auto tableElem : kLanguageTable)
    {
        if (tableElem.primaryLangId == PRIMARYLANGID(langId))
        {
            if ((tableElem.subLangId == SUBLANG_NEUTRAL) || (tableElem.subLangId == SUBLANGID(langId)))
            {
                _defaultInfo.language = tableElem.language;
                break;
            }
        }
    }

    _currentInfo = _defaultInfo;
}

}   // namespace MGL::System

#endif  // MGL_TARGET_WIN32

// vim: et ts=4 sw=4 sts=4
