// SPDX-License-Identifier: Zlib
/* ------------------------------------------------------------------------- */
/*!
 *  \file       mgl_task.cc
 *  \brief      MGL タスクシステム
 *  \date       Since: June 3, 2021. 11:55:39 JST.
 *  \author     Acerola
 */
/* ------------------------------------------------------------------------- */

#include <mgl/task/mgl_task.h>

namespace MGL::Task
{
/* ------------------------------------------------------------------------- */
/*!
 *  \brief      タスクシステムの初期化
 *  \param[in]  capacity            タスクの最大数
 *  \param[in]  executeSettings     実行設定の配列
 *  \retval     true                成功
 *  \retval     false               失敗
 */
/* ------------------------------------------------------------------------- */
bool Initialize(size_t capacity, const ExecuteSettingArray &executeSettings) noexcept
{
    if (NodeList::HasInstance())
    {
        return false;
    }

    auto &nodeList = NodeList::CreateInstance();
    
    if (executeSettings.empty())
    {
        auto defaultSettings = ExecuteSettingArray{
            {kExecuteStageUpdate},
            {kExecuteStageRender},
        };

        nodeList.Initialize(capacity, defaultSettings);
    }
    else
    {
        nodeList.Initialize(capacity, executeSettings);
    }
    
    return true;
}


/* ------------------------------------------------------------------------- */
/*!
 *  \brief      タスクを検索
 *  \param[in]  identifier  検索するID
 *  \return     見つかったタスクの弱参照ノードの配列
 */
/* ------------------------------------------------------------------------- */
STL::vector<WeakNode> Find(Identifier identifier) noexcept
{
    auto indexArray = NodeList::GetInstance().Find(identifier);

    STL::vector<WeakNode> result;
    result.reserve(indexArray.size());
    
    for (auto index : indexArray)
    {
        WeakNode weakNode(index);
        result.push_back(weakNode);
    }

    return result;
}

}   // namespace MGL::Task

// vim: et ts=4 sw=4 sts=4
